#
#                py-google-search  Copyright (C) 2018  Javinator9889                
#   This program comes with ABSOLUTELY NO WARRANTY; for details add the "-h" option.
#           This is free software, and you are welcome to redistribute it
#                 under certain conditions; type "-L" for details.
#


class InvalidTimeException(RuntimeError):
    def __init__(self, message: str = None):
        self.__message = message
        super().__init__(message)


class InvalidCombinationException(RuntimeError):
    def __init__(self, message: str = None):
        self.__message = message
        super().__init__(message)


class TimeCombinationNonValid(InvalidCombinationException):
    def __init__(self, message: str = None):
        super().__init__(message)


class MixedSearchException(InvalidCombinationException):
    def __init__(self, message: str = None):
        super().__init__(message)


class NullQueryError(RuntimeError):
    def __init__(self, message: str = None):
        self.__message = message
        super().__init__(message)


class GoogleOverloadedException(RuntimeError):
    def __init__(self, message: str = None):
        self.__message = message
        super().__init__(message)


class GoogleBlockingConnectionsError(RuntimeError):
    def __init__(self, message: str = None):
        self.__message = message
        super().__init__(message)



#
#                py-google-search  Copyright (C) 2018  Javinator9889
#   This program comes with ABSOLUTELY NO WARRANTY; for details add the "-h" option.
#           This is free software, and you are welcome to redistribute it
#                 under certain conditions; type "-L" for details.
#
import urllib.parse

from .url_constants import __google_base_url__, __google_url_modifiers__
from pyGle.values import *
from pyGle.values.InterfaceLanguages import InterfaceLanguages


class GoogleSearch:
    def __init__(self, query: str = None):
        self.query = query
        self.words_in_order = None
        self.or_words = None
        self.words_excluded = None
        self.number_of_results = None
        self.filetype = None
        self.sitesearch = None
        self.time_limit = None
        self.rights = None
        self.words_in_title = None
        self.words_in_body = None
        self.words_in_url = None
        self.words_in_anchor = None
        self.between_two_numbers = None
        self.synonym = None
        self.containing_two_words = None
        self.safe = None
        self.related = None
        self.linked = None
        self.personalized_search = None
        self.results_lang = None
        self.country = None
        self.document_country = None
        self.search_between_two_dates = None
        self.sort_by_update_time = None
        self.search_at_different_pages = False
        self.search_at = None
        self.start_position = None
        self.image_params = None
        self.patents_params = None
        self.shop_params = None
        self.interface_language = None
        self.book_params = None
        self.video_params = None

    def withQuery(self, query: str):
        self.query = query.replace(' ', '+')
        return self

    def withWordsInSpecificOrderSearch(self, words: list):
        self.words_in_order = '+'.join(words)
        return self

    def withOneOrMoreWordsInResult(self, words: list):
        self.or_words = '+'.join(words)
        return self

    def withExcludedWords(self, words: list):
        self.words_excluded = '+'.join(words)
        return self

    def withNumberOfResults(self, results: int = 10):
        self.number_of_results = str(results)
        return self

    def withFileSearchByFileType(self, filetype: str):
        self.filetype = filetype.replace(' ', '+')
        return self

    def withSiteSearch(self, site: str):
        self.sitesearch = site.replace(' ', '+')
        return self

    def withTimeLimit(self, time_limit: TimeLimit):
        self.time_limit = time_limit.getTimeLimit()
        return self

    def withRights(self, rights: Rights):
        self.rights = rights.getRights()
        return self

    def withTextInTitle(self, text: str):
        self.words_in_title = text.replace(' ', '+')
        return self

    def withTextInBody(self, text: str):
        self.words_in_body = text.replace(' ', '+')
        return self

    def withTextInUrl(self, text: str):
        self.words_in_url = text.replace(' ', '+')
        return self

    def withTextInAnchor(self, text: str):
        self.words_in_anchor = text.replace(' ', '+')
        return self

    def withResultBetweenTwoNumbers(self, first_number: float, second_number: float):
        self.between_two_numbers = [str(first_number), str(second_number)]
        return self

    def withRelatedPagesToADocument(self, document_url: str):
        self.related = document_url.replace(' ', '+')
        return self

    def withSynonymousSearchingToAWord(self, word: str):
        self.synonym = word.replace(' ', '+')
        return self

    def withContainingTwoTerms(self, first_term: str, second_term: str):
        self.containing_two_words = [first_term.replace(' ', '+'), second_term.replace(' ', '+')]
        return self

    def withSafeModeActivated(self):
        self.safe = __google_url_modifiers__["with_safe_active"]
        return self

    def withSafeModeDeactivated(self):
        self.safe = __google_url_modifiers__["with_safe_deactivated"]
        return self

    def withLinkingToUrl(self, url: str):
        self.linked = url.replace(' ', '+')
        return self

    def withPersonalizedSearch(self):
        self.personalized_search = __google_url_modifiers__["with_personalized_search"]
        return self

    def withNoPersonalizedSearch(self):
        self.personalized_search = __google_url_modifiers__["with_no_personalized_search"]
        return self

    def withResultsLanguage(self, lang: Languages):
        self.results_lang = lang.getLanguage()
        return self

    def withResultsAtCountry(self, country: Countries):
        self.country = country.getCountry()
        return self

    def withDocumentCountry(self, country: Countries):
        self.document_country = country.getCountry()
        return self

    def withSearchBetweenTwoDates(self, dates: Dates):
        self.search_between_two_dates = dates.getDates()
        return self

    def withSortByUpdateTime(self):
        self.sort_by_update_time = True
        return self

    def withSearchingAtDifferentGooglePages(self, google_pages: GooglePages):
        self.search_at_different_pages = True
        self.search_at = google_pages.getGooglePage()
        return self

    def withSearchStartPositionAt(self, index: int):
        self.start_position = str(index)
        return self

    def withImageParams(self, params: GoogleImages):
        params_dict = params.getImageParams()
        self.image_params = {}
        for key, value in params_dict.items():
            if value:
                self.image_params[key] = value
        return self

    def withPatentParams(self, params: GooglePatents):
        params_dict = params.getPatentModifiers()
        self.patents_params = {}
        for key, value in params_dict.items():
            if value:
                self.patents_params[key] = value
        return self

    def withShopOptions(self, params: GoogleShop):
        params_dict = params.getShopModifiers()
        self.shop_params = {}
        for key, value in params_dict.items():
            if value:
                self.shop_params[key] = value
        return self

    def withInterfaceLanguage(self, language: InterfaceLanguages):
        self.interface_language = language
        return self

    def withBookParams(self, params: GoogleBooks):
        params_dict = params.getBooksModifiers()
        self.book_params = {}
        for key, value in params_dict.items():
            if value:
                self.book_params[key] = value
        return self

    def withVideoParams(self, params: GoogleVideos):
        params_dict = params.getVideoModifiers()
        self.video_params = {}
        for key, value in params_dict.items():
            if value:
                self.video_params[key] = value
        return self


class URLBuilder:
    def __init__(self, google_search_params: GoogleSearch):
        self.params = google_search_params

    def __params_validator(self):
        from errors import TimeCombinationNonValid, MixedSearchException

        if self.params.search_between_two_dates and self.params.time_limit:
            raise TimeCombinationNonValid("You cannot search between two dates and a define a time limit")
        if self.params.search_at_different_pages:
            if self.params.search_at == __google_url_modifiers__["with_searching"]["books"]:
                if self.params.image_params or self.params.patents_params or self.params.shop_params \
                        or self.params.video_params:
                    raise MixedSearchException("You cannot combine a Google Books Search with non-proper params")
            elif self.params.search_at == __google_url_modifiers__["with_searching"]["images"]:
                if self.params.shop_params or self.params.patents_params or self.params.book_params \
                        or self.params.video_params:
                    raise MixedSearchException("You cannot combine a Google Images Search with non-proper params")
            elif self.params.search_at == __google_url_modifiers__["with_searching"]["news"]:
                if self.params.shop_params or self.params.patents_params or self.params.book_params or \
                        self.params.image_params or self.params.video_params:
                    raise MixedSearchException("You cannot combine a Google News Search with non-proper params")
            elif self.params.search_at == __google_url_modifiers__["with_searching"]["patents"]:
                if self.params.shop_params or self.params.image_params or self.params.book_params \
                        or self.params.video_params:
                    raise MixedSearchException("You cannot combine a Google Patents Search with non-proper params")
            elif self.params.search_at == __google_url_modifiers__["with_searching"]["shops"]:
                if self.params.patents_params or self.params.book_params or self.params.image_params \
                        or self.params.video_params:
                    raise MixedSearchException("You cannot combine a Google Shops Search with non-proper params")
            elif self.params.search_at == __google_url_modifiers__["with_searching"]["videos"]:
                if self.params.shop_params or self.params.patents_params or self.params.book_params or \
                        self.params.image_params:
                    raise MixedSearchException("You cannot combine a Google Videos Search with non-proper params")

    def build(self):
        # type: () -> tuple
        from errors import NullQueryError

        self.__params_validator()
        main_query = []
        if self.params.query:
            main_query.append(__google_url_modifiers__["query"].format(self.params.query))
        else:
            raise NullQueryError("At least \"query\" must be provided")
        if self.params.words_in_title:
            main_query.append(__google_url_modifiers__["with_text_in_title"].format(self.params.words_in_title))
        if self.params.words_in_body:
            main_query.append(__google_url_modifiers__["with_text_in_body"].format(self.params.words_in_body))
        if self.params.words_in_url:
            main_query.append(__google_url_modifiers__["with_text_in_url"].format(self.params.words_in_url))
        if self.params.words_in_anchor:
            main_query.append(
                __google_url_modifiers__["with_text_in_anchor"].format(self.params.words_in_anchor))
        if self.params.between_two_numbers:
            main_query.append(
                __google_url_modifiers__["between_two_numbers"].format(self.params.between_two_numbers[0],
                                                                       self.params.between_two_numbers[1]))
        if self.params.synonym:
            main_query.append(__google_url_modifiers__["with_synonymous_searching"].format(self.params.synonym))
        if self.params.containing_two_words:
            main_query.append(__google_url_modifiers__["with_containing_two_terms"].format(
                self.params.containing_two_words[0], self.params.containing_two_words[1]))

        extra_attributes_query = []
        tbs_attributes = []
        if self.params.words_in_order:
            extra_attributes_query.append(
                __google_url_modifiers__["in_order_displayed"].format(self.params.words_in_order))
        if self.params.or_words:
            extra_attributes_query.append(__google_url_modifiers__["or"].format(self.params.or_words))
        if self.params.words_excluded:
            extra_attributes_query.append(
                __google_url_modifiers__["exclude"].format(self.params.words_excluded))
        if self.params.number_of_results:
            extra_attributes_query.append(
                __google_url_modifiers__["number_of_results"].format(self.params.number_of_results))
        if self.params.filetype:
            extra_attributes_query.append(
                __google_url_modifiers__["with_extension"].format(self.params.filetype))
        if self.params.sitesearch:
            extra_attributes_query.append(__google_url_modifiers__["at_site"].format(self.params.sitesearch))
        if self.params.time_limit:
            extra_attributes_query.append(
                __google_url_modifiers__["with_time_limit"].format(self.params.time_limit))
        if self.params.rights:
            extra_attributes_query.append(__google_url_modifiers__["with_rights"].format(self.params.rights))
        if self.params.safe:
            extra_attributes_query.append(self.params.safe)
        if self.params.related:
            extra_attributes_query.append(
                __google_url_modifiers__["with_related_pages_to_a_doc"].format(self.params.related))
        if self.params.linked:
            extra_attributes_query.append(
                __google_url_modifiers__["with_linking_to_url"].format(self.params.linked))
        if self.params.personalized_search:
            extra_attributes_query.append(self.params.personalized_search)
        if self.params.results_lang:
            extra_attributes_query.append(
                __google_url_modifiers__["with_results_language"].format(self.params.results_lang))
        if self.params.interface_language:
            extra_attributes_query.append(
                __google_url_modifiers__["with_language_interface"].format(self.params.interface_language))
        if self.params.country:
            extra_attributes_query.append(
                __google_url_modifiers__["with_country_results"].format(self.params.country))
        if self.params.document_country:
            extra_attributes_query.append(
                __google_url_modifiers__["with_document_county"].format(self.params.document_country))
        if self.params.search_between_two_dates:
            if self.params.sort_by_update_time:
                tbs_attributes.append(
                    __google_url_modifiers__["with_search_between_two_dates_by_update_time"].format(
                        self.params.search_between_two_dates[0], self.params.search_between_two_dates[1]))
            else:
                tbs_attributes.append(__google_url_modifiers__["with_search_between_two_dates"].format(
                    self.params.search_between_two_dates[0], self.params.search_between_two_dates[1]))
        if self.params.sort_by_update_time and not self.params.search_between_two_dates:
            tbs_attributes.append(__google_url_modifiers__["with_search_by_update_time"])
        if self.params.search_at_different_pages:
            extra_attributes_query.append(
                __google_url_modifiers__["with_searching_at_different_google_pages"].format(
                    self.params.search_at))
        if self.params.start_position:
            extra_attributes_query.append(
                __google_url_modifiers__["with_starting_at_position"].format(self.params.start_position))
        if self.params.image_params:
            for key, value in self.params.image_params.items():
                tbs_attributes.append(value)
        if self.params.patents_params:
            for key, value in self.params.patents_params.items():
                tbs_attributes.append(value)
        if self.params.book_params:
            for key, value in self.params.book_params.items():
                tbs_attributes.append(value)
        if self.params.video_params:
            for key, value in self.params.video_params.items():
                tbs_attributes.append(value)
        if self.params.shop_params:
            try:
                if self.params.shop_params["min_price"] is not None:
                    tbs_attributes.append(__google_url_modifiers__["with_shop_params"]["with_custom_price"])
            except KeyError:
                try:
                    if self.params.shop_params["max_price"] is not None:
                        tbs_attributes.append(__google_url_modifiers__["with_shop_params"]["with_custom_price"])
                except KeyError:
                    pass
            for key, value in self.params.shop_params.items():
                tbs_attributes.append(value)

        final_query = '+'.join(main_query)
        final_attributes = '&'.join(extra_attributes_query)
        if len(tbs_attributes) > 0:
            extra_attributes = __google_url_modifiers__["with_extra_attributes"].format(','.join(tbs_attributes))
        else:
            extra_attributes = None
        if final_attributes and extra_attributes:
            attributes_query = final_query + '&' + final_attributes + '&' + extra_attributes
        elif final_attributes:
            attributes_query = final_query + '&' + final_attributes
        elif extra_attributes:
            attributes_query = final_query + '&' + extra_attributes
        else:
            attributes_query = final_query
        return urllib.parse.quote_plus(__google_base_url__ + attributes_query, safe="/?+&:=_.(|)*-%,~")



#
#                py-google-search  Copyright (C) 2018  Javinator9889
#   This program comes with ABSOLUTELY NO WARRANTY; for details add the "-h" option.
#           This is free software, and you are welcome to redistribute it
#                 under certain conditions; type "-L" for details.
#
from pyGle.url.url_constants import __google_url_modifiers__
from .AvailableLanguages import AvailableLanguages
from .AvailableCountries import AvailableCountries
from .OptionsForPictures import *
from .OptionsForPatents import *


class TimeLimit:
    def __init__(self):
        self.__time_limit = None
        self.__times = __google_url_modifiers__["with_time_limitation_for"]

    def setDay(self):
        self.__time_limit = self.__times["day"]

    def setWeek(self):
        self.__time_limit = self.__times["week"]

    def setMonth(self):
        self.__time_limit = self.__times["month"]

    def setYear(self):
        self.__time_limit = self.__times["year"]

    def setMonths(self, time: int):
        if time <= 0:
            raise ValueError("Time must be greater than '0'")
        else:
            self.__time_limit = self.__times["month"] + str(time)

    def getTimeLimit(self):
        # type: () -> str
        return self.__time_limit


class Rights:
    def __init__(self):
        self.__rights = None
        self.__available_rights = __google_url_modifiers__["with_rights_types"]

    def setFreeToUse(self):
        self.__rights = self.__available_rights["free_to_use"]

    def setFreeToUseCommerciallyAlso(self):
        self.__rights = self.__available_rights["free_to_use_com"]

    def setFreeToUseAndModify(self):
        self.__rights = self.__available_rights["free_to_use_and_modify"]

    def setFreeToUseAndModifyCommerciallyAlso(self):
        self.__rights = self.__available_rights["free_to_use_and_modify_com"]

    def getRights(self):
        # type: () -> str
        return self.__rights


class Languages:
    def __init__(self):
        self.__lang = None
        self.__available_lang = __google_url_modifiers__["with_languages"]

    def show_languages(self):
        for key, value in self.__available_lang.items():
            print(key)

    def setLanguage(self, language: AvailableLanguages):
        self.__lang = language

    def getLanguage(self):
        # type: () -> str
        return self.__lang


class Countries:
    def __init__(self):
        self.__country = None
        self.__available_countries = __google_url_modifiers__["with_countries"]

    def show_countries(self):
        for key, values in self.__available_countries.items():
            print(key)

    def setCountry(self, country: AvailableCountries):
        self.__country = country

    def getCountry(self):
        # type: () -> str
        return self.__country


class Dates:
    def __init__(self):
        self.__date_one = None
        self.__date_two = None

    def setFirstDate(self, day: int, month: int, year: int):
        from datetime import datetime

        self.__date_one = datetime(year, month, day).strftime("%d.%m.%Y")

    def setSecondDate(self, day: int, month: int, year: int):
        from datetime import datetime

        self.__date_two = datetime(year, month, day).strftime("%d.%m.%Y")

    def getDates(self):
        # type: () -> list
        return [self.__date_one, self.__date_two]


class GooglePages:
    def __init__(self):
        self.__google_page = None
        self.__available_google_pages = __google_url_modifiers__["with_searching"]

    def searchImage(self):
        self.__google_page = self.__available_google_pages["images"]

    def searchNews(self):
        self.__google_page = self.__available_google_pages["news"]

    def searchBooks(self):
        self.__google_page = self.__available_google_pages["books"]

    def searchShops(self):
        self.__google_page = self.__available_google_pages["shops"]

    def searchVideos(self):
        self.__google_page = self.__available_google_pages["videos"]

    def searchPatents(self):
        self.__google_page = self.__available_google_pages["patents"]

    def getGooglePage(self):
        # type: () -> str
        return self.__google_page


class GoogleImages:
    def __init__(self):
        self.__available_images_modifiers = __google_url_modifiers__["with_image_params"]
        self.__image_params = {
            "image_color": None,
            "color_type": None,
            "rights": None,
            "size": None,
            "type": None,
            "aspect_ratio": None,
            "image_format": None
        }

    def setColor(self, color: AvailableColors):
        self.__image_params["image_color"] = color

    def setColorType(self, color_type: AvailableColorsType):
        self.__image_params["color_type"] = color_type

    def setImageRights(self, rights: AvailableRights):
        self.__image_params["rights"] = rights

    def setImageSize(self, size: AvailableSizes):
        self.__image_params["size"] = size

    def setImageType(self, image_type: AvailableImagesTypes):
        self.__image_params["type"] = image_type

    def setAspectRatio(self, aspect_ratio: AvailableAspectRatios):
        self.__image_params["aspect_ratio"] = aspect_ratio

    def setImageFormat(self, image_format: AvailableImageFormats):
        self.__image_params["image_format"] = image_format

    def getImageParams(self) -> dict:
        return self.__image_params

    def showColors(self):
        self.__print_keys_in_dict("color")

    def showColorTypes(self):
        self.__print_keys_in_dict("color_type")

    def showImageRights(self):
        self.__print_keys_in_dict("image_rights")

    def showImageSizes(self):
        self.__print_keys_in_dict("image_size")

    def showImageTypes(self):
        self.__print_keys_in_dict("image_type")

    def showAspectRatios(self):
        self.__print_keys_in_dict("aspect_ratio")

    def showImageFormats(self):
        self.__print_keys_in_dict("format")

    def __print_keys_in_dict(self, source_key: str):
        for key, value in self.__available_images_modifiers[source_key].items():
            print(key)


class GooglePatents:
    def __init__(self):
        self.__available_patents_modifiers = __google_url_modifiers__["with_patents_params"]
        self.__patents_params = {
            "office_patents": None,
            "patent_status": None,
            "patent_type": None
        }

    def setOfficePatent(self, office_patent: OfficePatents):
        self.__patents_params["office_patents"] = office_patent

    def setPatentStatus(self, status: PatentStatus):
        self.__patents_params["patent_status"] = status

    def setPatentType(self, type: AvailablePatentTypes):
        self.__patents_params["patent_type"] = type

    def getPatentModifiers(self) -> dict:
        return self.__patents_params


class GoogleShop:
    def __init__(self):
        self.__available_shop_modifiers = __google_url_modifiers__["with_shop_params"]
        self.__shop_params = {
            "sort_order": None,
            "new_products": None,
            "max_price": None,
            "min_price": None
        }

    def onlyNewProducts(self):
        self.__shop_params["new_products"] = self.__available_shop_modifiers["only_new_products"]

    def orderByLowerToHigherPrice(self):
        self.__shop_params["sort_order"] = self.__available_shop_modifiers["sort_order_less_to_high"]

    def orderByHigherToLowerPrice(self):
        self.__shop_params["sort_order"] = self.__available_shop_modifiers["sort_order_high_to_less"]

    def orderByReviewScore(self):
        self.__shop_params["sort_order"] = self.__available_shop_modifiers["sort_order_reviews"]

    def withMinPrice(self, price: int):
        self.__shop_params["min_price"] = self.__available_shop_modifiers["with_min_price"].format(str(price))

    def withMaxPrice(self, price: int):
        self.__shop_params["max_price"] = self.__available_shop_modifiers["with_max_price"].format(str(price))

    def betweenTwoPrices(self, min_price: int, max_price: int):
        self.__shop_params["min_price"] = self.__available_shop_modifiers["with_min_price"].format(str(min_price))
        self.__shop_params["max_price"] = self.__available_shop_modifiers["with_max_price"].format(str(max_price))

    def getShopModifiers(self) -> dict:
        return self.__shop_params


class GoogleBooks:
    def __init__(self):
        self.__available_books_modifiers = __google_url_modifiers__["with_books_params"]
        self.__books_params = {
            "books_type": None,
            "with_searching": None
        }

    def searchOnlyBooksWithPreview(self):
        self.__books_params["books_type"] = self.__available_books_modifiers["books_with_preview"]

    def searchOnlyGoogleEBooks(self):
        self.__books_params["books_type"] = self.__available_books_modifiers["google_ebooks"]

    def searchOnlyFreeEBooks(self):
        self.__books_params["books_type"] = self.__available_books_modifiers["free_ebooks"]

    def searchOnlyBooks(self):
        self.__books_params["with_searching"] = self.__available_books_modifiers["with_searching_books"]

    def searchOnlyNewsPapers(self):
        self.__books_params["with_searching"] = self.__available_books_modifiers["with_searching_news"]

    def searchOnlyMagazines(self):
        self.__books_params["with_searching"] = self.__available_books_modifiers["with_searching_magazines"]

    def getBooksModifiers(self) -> dict:
        return self.__books_params


class GoogleVideos:
    def __init__(self):
        self.__available_video_modifiers = __google_url_modifiers__["with_video_params"]
        self.__video_params = {
            "duration": None,
            "high_quality": None,
            "subtitles": None
        }

    def withShortDuration(self):
        self.__video_params["duration"] = self.__available_video_modifiers["duration"]["short"]

    def withMediumDuration(self):
        self.__video_params["duration"] = self.__available_video_modifiers["duration"]["medium"]

    def withLongDuration(self):
        self.__video_params["duration"] = self.__available_video_modifiers["duration"]["long"]

    def withHighQualityVideos(self):
        self.__video_params["high_quality"] = self.__available_video_modifiers["high_quality"]

    def withSubtitles(self):
        self.__video_params["subtitles"] = self.__available_video_modifiers["with_subtitles"]

    def getVideoModifiers(self) -> dict:
        return self.__video_params



#
#           py-google-searchCopyright (C) 2018Javinator9889
# This program comes with ABSOLUTELY NO WARRANTY; for details add the-h option.
#      This is free software and you are welcome to redistribute it
#              under certain conditions; type -L for details.
#
from abc import ABCMeta as __ABCMeta


class AvailableColors(metaclass=__ABCMeta):
    Red = "ic:specific,isc:red"
    Orange = "ic:specific,isc:orange"
    Yellow = "ic:specific,isc:yellow"
    Green = "ic:specific,isc:green"
    Teal = "ic:specific,isc:teel"
    Blue = "ic:specific,isc:blue"
    Purple = "ic:specific,isc:purple"
    Pink = "ic:specific,isc:pink"
    White = "ic:specific,isc:white"
    Gray = "ic:specific,isc:gray"
    Black = "ic:specific,isc:black"
    Brown = "ic:specific,isc:brown"
    
    
class AvailableColorsType(metaclass=__ABCMeta):
    Full_color = "ic:color"
    Black_and_white = "ic:gray"
    Transparent = "ic:trans"
    

class AvailableRights(metaclass=__ABCMeta):
    Labeled_for_reuse_with_modifications = "sur:fmc"
    Labeled_for_reuse = "sur:fc"
    Labeled_for_noncommercial_reuse_with_modification = "sur:fm"
    Labeled_for_noncommercial_reuse = "sur:f"


class AvailableSizes(metaclass=__ABCMeta):
    Large = "isz:l"
    Medium = "isz:m"
    Icon = "isz:i"
    BiggerThan400x300 = "isz:lt,islt:qsvga"
    BiggerThan640x480 = "isz:lt,islt:vga"
    BiggerThan800x600 = "isz:lt,islt:svga"
    BiggerThan1024x768 = "visz:lt,islt:xga"
    BiggerThan2MP = "isz:lt,islt:2mp"
    BiggerThan4MP = "isz:lt,islt:4mp"
    BiggerThan6MP = "isz:lt,islt:6mp"
    BiggerThan8MP = "isz:lt,islt:8mp"
    BiggerThan10MP = "isz:lt,islt:10mp"
    BiggerThan12MP = "isz:lt,islt:12mp"
    BiggerThan15MP = "isz:lt,islt:15mp"
    BiggerThan20MP = "isz:lt,islt:20mp"
    BiggerThan40MP = "isz:lt,islt:40mp"
    BiggerThan70MP = "isz:lt,islt:70mp"
    

class AvailableImagesTypes(metaclass=__ABCMeta):
    Face = "itp:face"
    Photo = "itp:photo"
    Clip_art = "itp:clipart"
    Line_drawing = "itp:lineart"
    Animated = "itp:animated"


class AvailableAspectRatios(metaclass=__ABCMeta):
    Tall = "iar:t"
    Square = "iar:s"
    Wide = "iar:w"
    Panoramic = "iar:xw"
    

class AvailableImageFormats(metaclass=__ABCMeta):
    jpg = "ift:jpg"
    gif = "ift:gif"
    png = "ift:png"
    bmp = "ift:bmp"
    svg = "ift:svg"
    webp = "webp"
    ico = "ift:ico"
    


#
#                py-google-search  Copyright (C) 2018  Javinator9889                
#   This program comes with ABSOLUTELY NO WARRANTY; for details add the "-h" option.
#           This is free software, and you are welcome to redistribute it
#                 under certain conditions; type "-L" for details.
#
from abc import ABCMeta


class AvailableLanguages(metaclass=ABCMeta):
    Arabic = "lang_ar"
    Armenian = "lang_hy"
    Belorussian = "lang_be"
    Bulgarian = "lang_bg"
    Catalan = "lang_ca"
    Croatian = "lang_hr"
    Czech = "lang_cs"
    Danish = "lang_da"
    Dutch = "lang_nl"
    English = "lang_en"
    Esperanto = "lang_eo"
    Estonian = "lang_et"
    Filipino = "lang_tl"
    Finnish = "lang_fi"
    French = "lang_fr"
    German = "lang_de"
    Greek = "lang_el"
    Hebrew = "lang_iw"
    Hungarian = "lang_hu"
    Icelandic = "lang_is"
    Indonesian = "lang_id"
    Italian = "lang_it"
    Japanese = "lang_ja"
    Korean = "lang_ko"
    Latvian = "lang_lv"
    Lithuanian = "lang_lt"
    Norwegian = "lang_no"
    Persian = "lang_fa"
    Polish = "lang_pl"
    Portuguese = "lang_pt"
    Romanian = "lang_ro"
    Russian = "lang_ru"
    Serbian = "lang_sr"
    Slovak = "lang_sk"
    Slovenian = "lang_sl"
    Spanish = "lang_es"
    Swedish = "lang_sv"
    Thai = "lang_sv"
    Turkish = "lang_tr"
    Ukrainian = "lang_uk"
    Vietnamese = "lang_vi"
    ChineseSimplified = "lang_zh-CN"
    ChineseTraditional = "lang_zh-TW"



#
#                py-google-search  Copyright (C) 2018  Javinator9889
#   This program comes with ABSOLUTELY NO WARRANTY; for details add the "-h" option.
#           This is free software, and you are welcome to redistribute it
#                 under certain conditions; type "-L" for details.
#
import lxml
import urllib.request
import urllib.parse
import urllib.error
import ujson as json
import random
import time

from bs4 import BeautifulSoup
from concurrent.futures import Future, ThreadPoolExecutor
from multiprocessing import cpu_count

from pyGle.url import URLBuilder
from pyGle.url.url_constants import __user_agents__
from pyGle.errors import GoogleOverloadedException, GoogleBlockingConnectionsError


class BaseExtractor:
    def __init__(self, must_use_session: bool = False, with_history_enabled: bool = False):
        key = random.choice(list(__user_agents__.keys()))
        self.headers = {"User-Agent": __user_agents__[key]}
        self.cpu_count = cpu_count() * 2
        self.session_cookies = {"cookie": None} if must_use_session else {"cookie": "disabled"}
        self.history = [] if with_history_enabled else None
        self.url = None

    def extract_url(self, url: URLBuilder) -> Future:
        pass

    def obtain_html_object(self, url: URLBuilder) -> tuple:
        try:
            start_time = time.time()
            built_url = url.build()
            self.url = built_url
            request = urllib.request.Request(url=built_url, headers=self.headers)
            if self.session_cookies["cookie"] != "disabled" and self.session_cookies["cookie"] is not None:
                request.add_header("cookie", self.session_cookies["cookie"])
            web_content = urllib.request.urlopen(request)
            if self.session_cookies["cookie"] != "disabled":
                self.session_cookies["cookie"] = web_content.headers.get("Set-Cookie")
            requested_data = web_content.read().decode("utf-8")
            end_time = time.time()
            local_executor = ThreadPoolExecutor(max_workers=self.cpu_count)
            local_executor.submit(web_content.close)
            local_executor.shutdown(wait=False)
            return BeautifulSoup(requested_data, "lxml"), (end_time - start_time)
        except urllib.error.HTTPError as request_error:
            raise GoogleBlockingConnectionsError("It looks like Google is blocking errors.\n\t- Headers: "
                                                 + str(request_error.headers) + "\n\t- Error: "
                                                 + request_error.read().decode("utf-8"))

    def change_header(self):
        new_key = random.choice(list(__user_agents__.keys()))
        self.headers["User-Agent"] = __user_agents__[new_key]

    def getHistory(self) -> list:
        return self.history

    def getOverallTime(self) -> tuple:
        if self.history:
            time_amount = 0
            google_search_amount = 0
            parsing_elements_amount = 0
            length = len(self.history)
            for element in self.history:
                element_length = len(element) - 1
                time_amount += float(element[element_length]["stats"]["overall_time"].replace(" s", ''))
                google_search_amount += float(element[element_length]["stats"]["google_search_time"].replace(" s", ''))
                parsing_elements_amount += float(
                    element[element_length]["stats"]["parsing_page_time"].replace(" s", ''))
            return (time_amount / length), (google_search_amount / length), (parsing_elements_amount / length)
        else:
            return -1, -1, -1

    def printOverallTime(self):
        overall_time, google_search, parsing_elements = self.getOverallTime()
        if overall_time != -1:
            print("Total requests: " + str(len(self.history)))
            print("Overall time: " + str(overall_time) + " s")
            print("Google search overall time: " + str(google_search) + " s")
            print("Parsing elements overall time: " + str(parsing_elements) + " s")
        else:
            print("History is disabled or not petition has been done yet")

    @staticmethod
    def is_valid_url(url) -> bool:
        import re

        # From: https://stackoverflow.com/questions/7160737/python-how-to-validate-a-url-in-python-malformed-or-not
        regex = re.compile(
            r'^(?:http|ftp)s?://'  # http:// or https://
            r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain...
            r'localhost|'  # localhost...
            r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})'  # ...or ip
            r'(?::\d+)?'  # optional port
            r'(?:/?|[/?]\S+)$', re.IGNORECASE)
        return re.match(regex, url) is not None

    @staticmethod
    def cleanupString(text: str) -> str:
        import re

        return re.sub(u'[^a-zA-Z0-9áéíóúÁÉÍÓÚâêîôÂÊÎÔãõÃÕçÇñÑÄËÏÖÜäëïöü: \-.()/]', '', text)


class ImageExtractor(BaseExtractor):
    def __extractor(self, url: URLBuilder, start_time: float) -> list:
        html, search_time = super().obtain_html_object(url)
        images = []
        elements_start_time = time.time()
        for a in html.find_all("div", {"class": "rg_meta"}):
            image_properties = json.loads(a.get_text(strip=True))
            link = image_properties.get("ou", None)
            if link:
                image_format = image_properties.get("ity", "unknown")
                image_id = image_properties.get("id", "unknown")
                source = image_properties.get("isu", "unknown")
                width = image_properties.get("oh", "unknown")
                height = image_properties.get("ow", "unknown")
                web_page_name = image_properties.get("pt", "unknown")
                web_page_title = image_properties.get("s", "unknown")
                image_posted_on = image_properties.get("ru", "unknown")
                website_name = image_properties.get("st", "unknown")

                found_image_properties = {
                    "url": link,
                    "format": image_format,
                    "id": image_id,
                    "source": source,
                    "width": width,
                    "height": height,
                    "posted_on": image_posted_on,
                    "website_name": website_name,
                    "webpage_name": web_page_name,
                    "webpage_title": web_page_title
                }
                images.append(found_image_properties)
        elements_end_time = time.time()
        images.append({
            "stats": {
                "images_found": len(images),
                "overall_time": str((time.time() - start_time)) + " s",
                "google_search_time": str(search_time) + " s",
                "parsing_page_time": str((elements_end_time - elements_start_time)) + " s"
            },
            "url": self.url
        })
        if self.history is not None:
            self.history.append(images)
        super().change_header()
        return images

    def extract_url(self, url: URLBuilder) -> Future:
        start_time = time.time()
        executor = ThreadPoolExecutor(max_workers=self.cpu_count)
        future = executor.submit(self.__extractor, url, start_time)
        executor.shutdown(wait=False)
        return future


class SearchExtractor(BaseExtractor):
    @staticmethod
    def __get_web_page_title_link(web_section) -> tuple:
        try:
            search_title_object = web_section.find_all("h3", {"class": "r"})[0].find_all("a")
            link = search_title_object[0].get("href")
            web_page_title = search_title_object[0].string
        except IndexError:
            link = "unavailable"
            web_page_title = "unavailable"
        return link, web_page_title

    @staticmethod
    def __obtain_detailed_section(web_section) -> BeautifulSoup:
        try:
            return web_section.find_all("div", {"class": "s"})[0]
        except IndexError:
            return None

    @staticmethod
    def __obtain_web_cache(detailed_section) -> str:
        try:
            web_cache_version = detailed_section.find_all("li",
                                                          {"class": "action-menu-item ab_dropdownitem",
                                                           "role": "menuitem"})[0]
            web_cache_link = web_cache_version.find_all("a", {"class": "fl"})[0].get("href")
        except IndexError:
            web_cache_link = "unavailable"
        return web_cache_link

    @staticmethod
    def __obtain_date(detailed_section) -> str:
        try:
            date = detailed_section.find_all("span", {"class": "f"})[0].string
        except IndexError:
            date = "unavailable"
        return date

    @staticmethod
    def __obtain_related_pages(detailed_section) -> list:
        try:
            related_pages = []
            for related_page in detailed_section.find_all("div", {"class": "VNLkW"}):
                page_info = {}
                related_link = related_page.find_all("a", {"class": "fl"})[0].get("href")
                related_title = related_page.find_all("a", {"class": "fl"})[0].get_text(strip=True)
                related_date = related_page.find_all("div", {"class": "G1Rrjc"})[1].get_text(strip=True)
                if related_link:
                    page_info["link"] = related_link
                if related_title:
                    page_info["title"] = related_title
                if related_date:
                    page_info["date"] = related_date
                if len(page_info) != 0:
                    related_pages.append(page_info)
            if len(related_pages) == 0:
                related_pages = None
        except IndexError:
            related_pages = None
        return related_pages

    @staticmethod
    def __obtain_related_search(origin_html):
        try:
            related_search_result = origin_html.find_all("div", {"class": "card-section"})[0]
            related_search = []
            for other_search in related_search_result.find_all("a"):
                related_search.append(other_search.get_text(strip=True))
        except IndexError:
            related_search = "unavailable"
        return related_search

    @staticmethod
    def __obtain_stats(origin_html):
        try:
            stats = origin_html.find_all("div", {"id": "resultStats"})[0].get_text(strip=True)
        except IndexError:
            stats = "unavailable"
        return stats

    @staticmethod
    def __find_pages_in_selection(web_section) -> list:
        try:
            in_search_page = web_section.find_all("table", {"class": "nrgt"})[0]
            pages = in_search_page.find_all("div", {"class": "sld vsc"})
            results = []
            for page in pages:
                title_section = page.find("h3", {"class": "r"}).find("a", {"class": "l"})
                try:
                    title = title_section.get_text(strip=True).strip()
                    link = title_section.get("href").strip()
                    description = page.find("div", {"class": "s"}).find("div",
                                                                        {"class": "st"}).get_text(strip=True).strip()
                    results.append({
                        "title": title,
                        "link": link,
                        "description": description
                    })
                except AttributeError:
                    pass
            return results
        except IndexError:
            return []

    def __extractor(self, url: URLBuilder, start_time: float) -> list:
        html, search_time = super().obtain_html_object(url)
        search_results = []
        elements_start_time = time.time()
        results_areas = html.find_all("div", {"class": "bkWMgd"})
        for section in results_areas:
            found_results = section.find_all("div", {"class": "g"})
            for result in found_results:
                link, web_page_title = self.__get_web_page_title_link(result)
                more_info = self.__obtain_detailed_section(result)
                if more_info is None:
                    break
                web_cache_link = self.__obtain_web_cache(more_info)
                inside_pages = self.__find_pages_in_selection(result)
                try:
                    search_more_data = more_info.find_all("span", {"class": "st"})[0]
                    date = self.__obtain_date(search_more_data)
                    related_pages = self.__obtain_related_pages(more_info)
                    description = search_more_data.get_text(strip=True).replace(date, '', 1)
                except IndexError:
                    date = "unavailable"
                    related_pages = "unavailable"
                    description = "unavailable"
                current_results = {
                    "link": link,
                    "title": web_page_title,
                    "cached_version": web_cache_link,
                    "date": date.replace(" - ", ''),
                    "description": self.cleanupString(description)
                }
                if related_pages:
                    current_results["related_pages"] = related_pages
                if len(inside_pages) != 0:
                    current_results["inside_pages"] = inside_pages
                search_results.append(current_results)
        related_search = self.__obtain_related_search(html)
        stats = self.__obtain_stats(html)
        elements_end_time = time.time()
        search_results.append({"how_many_results": len(search_results),
                               "related_search": related_search,
                               "google_stats": stats,
                               "stats": {
                                   "overall_time": str((time.time() - start_time)) + " s",
                                   "google_search_time": str(search_time) + " s",
                                   "parsing_page_time": str((elements_end_time - elements_start_time)) + " s"
                               },
                               "url": self.url})
        if self.history is not None:
            self.history.append(search_results)
        super().change_header()
        return search_results

    def extract_url(self, url: URLBuilder) -> Future:
        start_time = time.time()
        executor = ThreadPoolExecutor(max_workers=self.cpu_count)
        future = executor.submit(self.__extractor, url, start_time)
        executor.shutdown(wait=False)
        return future


class NewsExtractor(BaseExtractor):
    @staticmethod
    def __obtain_thumbnail(picture_class) -> str:
        try:
            a_section = picture_class.find("a", {"class": "top NQHJEb dfhHve"})
            image_found = a_section.find("img", {"class": "th BbeB2d"}).get("src")
            return image_found if NewsExtractor.is_valid_url(image_found) else "base64 image - URL not available"
        except AttributeError:
            return "unavailable"

    @staticmethod
    def __get_title_link(html) -> tuple:
        try:
            header = html.find_all("h3", {"class": "r dO0Ag"})[0].find_all("a")[0]
            link = header.get("href")
            title = header.get_text(strip=True)
        except IndexError:
            link = "unavailable"
            title = "unavailable"
        return link, title

    @staticmethod
    def __obtain_publisher_date_extra(main_content) -> tuple:
        publisher = main_content.find("span", {"class": "xQ82C e8fRJf"}).get_text(strip=True)
        if not publisher:
            publisher = "unavailable"
        date = main_content.find("span", {"class": "f nsa fwzPFf"}).get_text(strip=True)
        if not date:
            date = "unavailable"
        extra_data = main_content.find("span", {"class": "HgetDe DwKiF"})
        extra = extra_data.get_text(strip=True) if extra_data else None
        return publisher, date, extra

    def __obtain_description(self, main_content) -> str:
        description = main_content.get_text(strip=True)
        return self.cleanupString(description) if description else "unavailable"

    @staticmethod
    def __obtain_related_articles(section) -> list:
        found_articles = []
        articles = section.find_all("div", class_="card-section")
        for article in articles:
            article_attributes = {}
            try:
                title_data = article.find("a", {"class": "RTNUJf"})
                title = title_data.get_text(strip=True)
                link = title_data.get("href")
                publisher = article.find("span", class_="xQ82C").get_text(strip=True)
                date = article.find("span", class_="fwzPFf").get_text(strip=True)
                extra_data = article.find("span", class_="HgetDe")
                extra = extra_data.get_text(strip=True) if extra_data else None
                article_attributes["title"] = title if title else "unavailable"
                article_attributes["link"] = link if link else "unavailable"
                article_attributes["publisher"] = publisher if publisher else "unavailable"
                article_attributes["date"] = date if date else "unavailable"
                if extra:
                    article_attributes["extra"] = extra
                found_articles.append(article_attributes)
            except AttributeError:
                pass
        return found_articles

    @staticmethod
    def __obtain_stats(origin_html):
        try:
            stats = origin_html.find_all("div", {"id": "resultStats"})[0].get_text(strip=True)
        except IndexError:
            stats = "unavailable"
        return stats

    def __extractor(self, url: URLBuilder, start_time: float) -> list:
        html, search_time = super().obtain_html_object(url)
        news_results = []
        elements_start_time = time.time()
        results_area = html.find_all("div", {"id": "ires"})[0]
        found_results = results_area.find_all("div", {"class": "g"})
        for result in found_results:
            main_content = result.find_all("div", {"class": "gG0TJc"})[0]
            thumbnail = self.__obtain_thumbnail(result.find("div", class_="ts"))
            link, title = self.__get_title_link(main_content)
            publisher, date, extra = self.__obtain_publisher_date_extra(main_content.find("div", {"class": "slp"}))
            description = self.__obtain_description(main_content.find("div", {"class": "st"}))
            related_articles = self.__obtain_related_articles(result)
            result_data = {
                "title": title,
                "link": link,
                "thumbnail": thumbnail,
                "publisher": publisher,
                "date": date,
                "extra": extra,
                "description": description,
                "related_articles": related_articles
            }
            if not extra:
                result_data.pop("extra", None)
            news_results.append(result_data)
        stats = self.__obtain_stats(html)
        elements_end_time = time.time()
        news_results.append({"how_many_results": len(news_results),
                             "google_stats": stats,
                             "stats": {
                                 "overall_time": str((time.time() - start_time)) + " s",
                                 "google_search_time": str(search_time) + " s",
                                 "parsing_page_time": str((elements_end_time - elements_start_time)) + " s"
                             },
                             "url": self.url})
        if self.history is not None:
            self.history.append(news_results)
        super().change_header()
        return news_results

    def extract_url(self, url: URLBuilder) -> Future:
        start_time = time.time()
        executor = ThreadPoolExecutor(max_workers=self.cpu_count)
        future = executor.submit(self.__extractor, url, start_time)
        executor.shutdown(wait=False)
        return future


class VideoExtractor(BaseExtractor):
    @staticmethod
    def __get_web_page_title_link(web_section) -> tuple:
        try:
            search_title_object = web_section.find_all("h3", {"class": "r"})[0].find_all("a")
            link = search_title_object[0].get("href")
            web_page_title = search_title_object[0].string
        except IndexError:
            link = "unavailable"
            web_page_title = "unavailable"
        return link, web_page_title

    @staticmethod
    def __obtain_detailed_section(web_section) -> BeautifulSoup:
        try:
            return web_section.find_all("div", {"class": "s"})[0]
        except IndexError:
            raise GoogleOverloadedException("It looks like Google is blocking your requests. Try enabling the "
                                            "proxy mode or wait for a few minutes")

    @staticmethod
    def __obtain_thumbnail(web_section) -> str:
        try:
            thumbnail_section = web_section.find("div", class_="N3nEGc").find("a").find("g-img").find("img")
            return thumbnail_section.get("src")
        except AttributeError:
            return "unavailable"

    @staticmethod
    def __obtain_duration(web_section) -> str:
        try:
            thumbnail_section = web_section.find("div", class_="N3nEGc").find("a").find("span", {"class": "vdur"})
            return thumbnail_section.get_text(strip=True).replace("▶", '').strip()
        except AttributeError:
            return "unavailable"

    @staticmethod
    def __obtain_location(web_section) -> str:
        try:
            data_section = web_section.find("div", class_="hJND5c").find("cite")
            return data_section.get_text(strip=True)
        except AttributeError:
            return "unavailable"

    @staticmethod
    def __obtain_date(web_section) -> str:
        try:
            date_section = web_section.find("div", class_="slp f")
            return date_section.get_text(strip=True)
        except AttributeError:
            return "unavailable"

    def __obtain_description(self, web_section) -> str:
        try:
            desc_section = web_section.find("span", {"class": "st"})
            return self.cleanupString(desc_section.get_text(strip=True))
        except AttributeError:
            return "unavailable"

    @staticmethod
    def __obtain_stats(origin_html):
        try:
            stats = origin_html.find_all("div", {"id": "resultStats"})[0].get_text(strip=True)
        except IndexError:
            stats = "unavailable"
        return stats

    def __extractor(self, url: URLBuilder, start_time: float) -> list:
        html, search_time = super().obtain_html_object(url)
        vid_results = []
        elements_start_time = time.time()
        results_areas = html.find_all("div", {"class": "srg"})
        for section in results_areas:
            found_results = section.find_all("div", {"class": "g"})
            for result in found_results:
                link, title = self.__get_web_page_title_link(result)
                more_info = self.__obtain_detailed_section(result)
                thumbnail = self.__obtain_thumbnail(more_info)
                duration = self.__obtain_duration(more_info)
                date = self.__obtain_date(more_info)
                description = self.__obtain_description(more_info)
                vid_results.append({
                    "title": title,
                    "link": link,
                    "description": description,
                    "date": date,
                    "duration": duration,
                    "thumbnail": thumbnail
                })
        stats = self.__obtain_stats(html)
        elements_end_time = time.time()
        vid_results.append({"how_many_results": len(vid_results),
                            "google_stats": stats,
                            "stats": {
                                "overall_time": str((time.time() - start_time)) + " s",
                                "google_search_time": str(search_time) + " s",
                                "parsing_page_time": str((elements_end_time - elements_start_time)) + " s"
                            },
                            "url": self.url})
        if self.history is not None:
            self.history.append(vid_results)
        super().change_header()
        return vid_results

    def extract_url(self, url: URLBuilder) -> Future:
        start_time = time.time()
        executor = ThreadPoolExecutor(max_workers=self.cpu_count)
        future = executor.submit(self.__extractor, url, start_time)
        executor.shutdown(wait=False)
        return future


class PatentExtractor(BaseExtractor):
    @staticmethod
    def __get_title_link(current_value) -> tuple:
        try:
            section = current_value.find("h3", {"class": "r"}).find("a")
            link = section.get("href")
            title = section.get_text(strip=True).strip()
        except AttributeError:
            link = "unavailable"
            title = "unavailable"
        return title, link

    @staticmethod
    def __obtain_website(details_section) -> str:
        try:
            section = details_section.find_all("cite")[0]
            cite = section.get_text(strip=True).strip()
        except IndexError:
            cite = "unavailable"
        return cite

    def __obtain_description(self, details_section) -> str:
        try:
            section = details_section.find("span", {"class": "st"})
            description = section.get_text(strip=True).strip()
        except AttributeError:
            description = "unavailable"
        return self.cleanupString(description)

    def __obtain_patent_image(self, details_section) -> str:
        try:
            section = details_section.find("img", {"class": "rISBZc M4dUYb"})
            image_link = section.get("src")
            return image_link if self.is_valid_url(image_link) else "base64 image - URL not available"
        except AttributeError:
            return "unavailable"

    def __get_patent_extras(self, details_section) -> tuple:
        import re
        try:
            section = details_section.find("div", {"class": "slp f"})
            first_parts = section.get_text(strip=True)
            regex_list = re.split(pattern="-(\xa0|\s)", string=first_parts,
                                  flags=re.IGNORECASE | re.MULTILINE | re.ASCII)
            parts = [x for x in regex_list if x != ' ']
            for i in range(len(parts)):
                parts[i] = self.cleanupString(parts[i])
            return parts[0][:-1], parts[1][:-1], parts[2][:-1], parts[3], parts[4]
        except (AttributeError, IndexError):
            return "unavailable", "unavailable", "unavailable", "unavailable", "unavailable"

    @staticmethod
    def __get_patent_description_related_forum(details_section) -> tuple:
        try:
            section = details_section.find("div", {"class": "osl"})
            parts = section.find_all("a")
            return parts[0].get("href"), parts[1].get("href"), parts[2].get("href")
        except (AttributeError, IndexError):
            return "unavailable", "unavailable", "unavailable"

    @staticmethod
    def __obtain_stats(origin_html):
        try:
            stats = origin_html.find_all("div", {"id": "resultStats"})[0].get_text(strip=True)
        except IndexError:
            stats = "unavailable"
        return stats

    def __extractor(self, url: URLBuilder, start_time: float) -> list:
        html, search_time = super().obtain_html_object(url)
        patents_results = []
        elements_start_time = time.time()
        results_areas = html.find_all("div", {"id": "ires"})
        for section in results_areas:
            found_results = section.find_all("div", {"class": "g"})
            for result in found_results:
                current_value = result.find("div", {"class": "rc"})
                title, link = self.__get_title_link(current_value)
                details_section = current_value.find("div", {"class": "s"})
                cite = self.__obtain_website(details_section)
                description = self.__obtain_description(details_section)
                patent_image = self.__obtain_patent_image(details_section)
                status, presentation_date, publication_date, inventor, assignee = \
                    self.__get_patent_extras(details_section)
                general_description, related_patents, forum = \
                    self.__get_patent_description_related_forum(details_section)
                patents_results.append({
                    "title": title,
                    "link": link,
                    "cite": cite,
                    "description": description,
                    "image": patent_image,
                    "status": status,
                    "presentation_date": presentation_date,
                    "publication_date": publication_date,
                    "inventor": inventor,
                    "assignee": assignee,
                    "general_description": general_description,
                    "related_patents": related_patents,
                    "forum": forum
                })
        stats = self.__obtain_stats(html)
        elements_end_time = time.time()
        patents_results.append({"how_many_results": len(patents_results),
                                "google_stats": stats,
                                "stats": {
                                    "overall_time": str((time.time() - start_time)) + " s",
                                    "google_search_time": str(search_time) + " s",
                                    "parsing_page_time": str((elements_end_time - elements_start_time)) + " s"
                                },
                                "url": self.url})
        if self.history is not None:
            self.history.append(patents_results)
        super().change_header()
        return patents_results

    def extract_url(self, url: URLBuilder) -> Future:
        start_time = time.time()
        executor = ThreadPoolExecutor(max_workers=self.cpu_count)
        future = executor.submit(self.__extractor, url, start_time)
        executor.shutdown(wait=False)
        return future


class ShopExtractor(BaseExtractor):
    @staticmethod
    def __find_thumbnail(result) -> str:
        img = result.find("div", {"class": "JRlvE XNeeld"}).find("img")
        return img.get("src")

    @staticmethod
    def __find_link_title(extra_data) -> tuple:
        try:
            section = extra_data.find("div", {"class": "eIuuYe"}).find("a")
            link = "https://google.com" + section.get("href")
            title = section.get_text(strip=True).strip()
        except AttributeError:
            link = "unavailable"
            title = "unavailable"
        return link, title

    @staticmethod
    def __find_price(extra_data) -> str:
        try:
            section = extra_data.find("div", {"class": "mQ35Be"}).find("span", {"class": "O8U6h"})
            return section.get_text(strip=True).strip().replace(u"\xa0", " ")
        except AttributeError:
            return "unavailable"

    @staticmethod
    def __find_description(extra_data) -> str:
        try:
            section = extra_data.find_all("div", {"class": "na4ICd"})
            description = None
            for result in section:
                if description:
                    description = description + "\n" + result.get_text(strip=True).strip()
                else:
                    description = result.get_text(strip=True).strip()
            return description.replace(u"\xa0", " ")
        except AttributeError:
            return "unavailable"

    @staticmethod
    def __find_reviews_score(extra_data) -> str:
        try:
            section = extra_data.find("div", {"class": "vq3ore"})
            return section.get("aria-label").replace(u"\xa0", " ")
        except AttributeError:
            return "unavailable"

    def __extractor(self, url: URLBuilder, start_time: float) -> list:
        html, search_time = super().obtain_html_object(url)
        shop_results = []
        elements_start_time = time.time()
        results_areas = html.find_all("div", {"id": "ires"})
        for section in results_areas:
            found_results = section.find_all("div", {"class": "sh-dlr__list-result"})
            for result in found_results:
                thumbnail = self.__find_thumbnail(result)
                extra_data = result.find("div", {"class": "ZGFjDb"})
                link, title = self.__find_link_title(extra_data)
                price = self.__find_price(extra_data)
                score = self.__find_reviews_score(extra_data)
                description = self.__find_description(extra_data)
                shop_results.append({
                    "title": title,
                    "link": link,
                    "thumbnail": thumbnail,
                    "price": price,
                    "score": score,
                    "description": description
                })
        elements_end_time = time.time()
        shop_results.append({"how_many_results": len(shop_results),
                             "stats": {
                                 "overall_time": str((time.time() - start_time)) + " s",
                                 "google_search_time": str(search_time) + " s",
                                 "parsing_page_time": str((elements_end_time - elements_start_time)) + " s"
                             },
                             "url": self.url})
        if self.history is not None:
            self.history.append(shop_results)
        super().change_header()
        return shop_results

    def extract_url(self, url: URLBuilder) -> Future:
        start_time = time.time()
        executor = ThreadPoolExecutor(max_workers=self.cpu_count)
        future = executor.submit(self.__extractor, url, start_time)
        executor.shutdown(wait=False)
        return future


class BookExtractor(BaseExtractor):
    @staticmethod
    def __get_book_title_link(web_section) -> tuple:
        try:
            search_title_object = web_section.find_all("h3", {"class": "r"})[0].find_all("a")
            link = search_title_object[0].get("href")
            web_page_title = search_title_object[0].string
        except IndexError:
            link = "unavailable"
            web_page_title = "unavailable"
        return link, web_page_title

    @staticmethod
    def __obtain_detailed_section(web_section) -> BeautifulSoup:
        try:
            return web_section.find_all("div", {"class": "s"})[0]
        except IndexError:
            raise GoogleOverloadedException("It looks like Google is blocking your requests. Try enabling the "
                                            "proxy mode or wait for a few minutes")

    @staticmethod
    def __obtain_thumbnail(web_section) -> str:
        try:
            thumbnail_section = web_section.find("div", class_="N3nEGc").find("a").find("g-img").find("img")
            return thumbnail_section.get("src")
        except AttributeError:
            return "unavailable"

    @staticmethod
    def __obtain_google_books_url(detailed_section) -> str:
        try:
            url_section = detailed_section.find("div", {"class": "f hJND5c TbwUpd"}).find("cite", {"class": "iUh30"})
            return url_section.get_text(strip=True).strip()
        except AttributeError:
            return "unavailable"

    def __get_books_extras(self, details_section) -> tuple:
        try:
            section = details_section.find("div", {"class": "slp f"})
            parts = section.get_text(strip=True).strip().split("-")
            for i in range(len(parts)):
                parts[i] = self.cleanupString(parts[i])
            first_part = parts[0]
            second_part = parts[1][1:][:-1]
            try:
                int(second_part)
                return first_part, second_part
            except ValueError:
                try:
                    second_part = "unavailable"
                    int(first_part)
                    return second_part, first_part
                except ValueError:
                    return "unavailable", "unavailable"
        except (AttributeError, IndexError):
            return "unavailable", "unavailable"

    @staticmethod
    def __get_book_description(details_section) -> str:
        try:
            section = details_section.find("span", {"class": "st"})
            return section.get_text(strip=True).strip().replace(u"\xa0", " ")
        except AttributeError:
            return "unavailable"

    @staticmethod
    def __obtain_stats(origin_html):
        try:
            stats = origin_html.find_all("div", {"id": "resultStats"})[0].get_text(strip=True)
        except IndexError:
            stats = "unavailable"
        return stats

    def __extractor(self, url: URLBuilder, start_time: float) -> list:
        html, search_time = super().obtain_html_object(url)
        book_results = []
        elements_start_time = time.time()
        results_areas = html.find_all("div", {"class": "srg"})
        for section in results_areas:
            found_results = section.find_all("div", {"class": "g"})
            for result in found_results:
                link, title = self.__get_book_title_link(result)
                more_info = self.__obtain_detailed_section(result)
                thumbnail = self.__obtain_thumbnail(more_info)
                description = self.__get_book_description(more_info)
                author, age = self.__get_books_extras(more_info)
                book_url = self.__obtain_google_books_url(more_info)
                book_results.append({
                    "title": title,
                    "link": link,
                    "description": description,
                    "book_url": book_url,
                    "thumbnail": thumbnail,
                    "publisher": {
                        "age": age,
                        "author": author
                    }
                })
        stats = self.__obtain_stats(html)
        elements_end_time = time.time()
        book_results.append({"how_many_results": len(book_results),
                             "google_stats": stats,
                             "stats": {
                                 "overall_time": str((time.time() - start_time)) + " s",
                                 "google_search_time": str(search_time) + " s",
                                 "parsing_page_time": str((elements_end_time - elements_start_time)) + " s"
                             },
                             "url": self.url})
        if self.history is not None:
            self.history.append(book_results)
        super().change_header()
        return book_results

    def extract_url(self, url: URLBuilder) -> Future:
        start_time = time.time()
        executor = ThreadPoolExecutor(max_workers=self.cpu_count)
        future = executor.submit(self.__extractor, url, start_time)
        executor.shutdown(wait=False)
        return future



#
#                py-google-search  Copyright (C) 2018  Javinator9889                
#   This program comes with ABSOLUTELY NO WARRANTY; for details add the "-h" option.
#           This is free software, and you are welcome to redistribute it
#                 under certain conditions; type "-L" for details.
#
from abc import ABCMeta


class OfficePatents(metaclass=ABCMeta):
    USA = "ptso:us"
    Europe = "ptso:ep"
    International = "ptso:wo"
    China = "ptso:cn"
    Germany = "ptso:de"
    Canada = "ptso:ca"
    

class PatentStatus(metaclass=ABCMeta):
    Applications = "ptss:a"
    IssuedPatents = "ptss:g"


class AvailablePatentTypes(metaclass=ABCMeta):
    Utility = "ptst:u"
    Design = "ptst:d"
    Plant = "ptst:pp"
    DefensivePublication = "ptst:t"
    AdditionalImprovement = "ptst:ai"
    StatutoryInventionRegistration = "ptst:h"
    


#
#                py-google-search  Copyright (C) 2018  Javinator9889                
#   This program comes with ABSOLUTELY NO WARRANTY; for details add the "-h" option.
#           This is free software, and you are welcome to redistribute it
#                 under certain conditions; type "-L" for details.
#
from pyGle.errors import InvalidTimeException


def check_time_limit(input_value: str):
    # type: () -> bool

    if len(input_value) >= 2:
        if input_value[0] == 'm':
            try:
                duration = int(input_value[1:])
                if duration <= 0:
                    __throw_invalid_time_exception(input_value)
            except ValueError:
                __throw_invalid_time_exception(input_value)
        else:
            __throw_invalid_time_exception(input_value)
    elif len(input_value) == 0:
        __throw_invalid_time_exception(input_value)
    else:
        if any(time in input_value for time in ['d', 'w', 'm', 'y']):
            return True
        else:
            __throw_invalid_time_exception(input_value)


def __throw_invalid_time_exception(input_value: str):
    raise InvalidTimeException("Unexpected value: waiting for: \n\t- d\n\t- w\n\t- m\n\t- y\n\t- m[NUMBER],\n got:"
                               " " + input_value)



#
#                py-google-search  Copyright (C) 2018  Javinator9889                
#   This program comes with ABSOLUTELY NO WARRANTY; for details add the "-h" option.
#           This is free software, and you are welcome to redistribute it
#                 under certain conditions; type "-L" for details.
#
from abc import ABCMeta as __ABCMeta


class InterfaceLanguages(metaclass=__ABCMeta):
    Afrikaans = "af"
    Luo = "ach"
    Akan = "ak"
    Amharic = "am"
    Arabic = "ar"
    Azerbaijani = "az"
    Belorussian = "be"
    Bemba = "bem"
    Bulgarian = "bg"
    Bihari = "bh"
    Bengali = "bn"
    Armenian = "hy"
    Basque = "eu"
    Bork = "xx-bork"
    Bosnian = "bs"
    Cambodian = "km"
    Catalan = "ca"
    Cherokee = "chr"
    Chichewa = "ny"
    ChineseSimplified = "zh-CN"
    ChineseTraditional = "zh-TW"
    Corsican = "co"
    Croatian = "hr"
    Czech = "cs"
    Danish = "da"
    Dutch = "nl"
    ElmerFudd = "xx-elmer"
    English = "en"
    Esperanto = "eo"
    Estonian = "et"
    Ewe = "ee"
    Faroese = "fo"
    Filipino = "tl"
    Finnish = "fi"
    French = "fr"
    Frisian = "fy"
    Ga = "gaa"
    Galician = "gl"
    Georgian = "ka"
    German = "de"
    Greek = "el"
    Guarani = "gn"
    Gujarati = "gu"
    Hacker = "xx-hacker"
    HaitianCreole = "ht"
    Hausa = "ha"
    Hawaiian = "haw"
    Hebrew = "iw"
    Hindi = "hi"
    Hungarian = "hu"
    Icelandic = "is"
    Igbo = "ig"
    Indonesian = "id"
    Interlingua = "ia"
    Irish = "ga"
    Italian = "it"
    Japanese = "ja"
    Javanese = "jw"
    Kannada = "kn"
    Kazakh = "kk"
    Kinyarwanda = "rw"
    Kirundi = "rn"
    Klingon = "xx-klingon"
    Kongo = "kg"
    Korean = "ko"
    KrioSierraLeone = "kri"
    Kurdish = "ku"
    KurdishSorani = "ckb"
    Kyrgyz = "ky"
    Laothian = "lo"
    Latin = "la"
    Latvian = "lv"
    Lingala = "ln"
    Lithuanian = "lt"
    Lozi = "loz"
    Luganda = "lg"
    Macedonian = "mk"
    Malagasy = "mg"
    Malay = "ms"
    Malayalam = "ml"
    Maltese = "mt"
    Maori = "mi"
    Marathi = "mr"
    MauritianCreole = "mfe"
    Moldavian = "mo"
    Mongolian = "mn"
    Montenegrin = "sr-ME"
    Nepali = "ne"
    NigerianPidgin = "pcm"
    NorthernSotho = "nso"
    Norwegian = "no"
    NorwegianNynorsk = "nn"
    Occitan = "oc"
    Oriya = "or"
    Oromo = "ps"
    Pashto = "fa"
    Persian = "fa"
    Pirate = "xx-pirate"
    Polish = "pl"
    PortugueseBrazil = "pt-BR"
    PortuguesePortugal = "pt-PT"
    Punjabi = "pa"
    Quechua = "qu"
    Romanian = "ro"
    Romansh = "rm"
    Runyakitara = "nyn"
    Russian = "ru"
    ScotsGaelic = "gd"
    Serbian = "sr"
    SerboCroatian = "sh"
    Sesotho = "st"
    Setswana = "tn"
    SeychelloisCreole = "crs"
    Shona = "sn"
    Sindhi = "sd"
    Sinhalese = "si"
    Slovak = "sk"
    Slovenian = "sl"
    Somali = "so"
    Spanish = "es"
    SpanishLatinAmerican = "es-419"
    Sundanese = "su"
    Swahili = "sw"
    Swedish = "sv"
    Tajik = "tg"
    Tamil = "ta"
    Tatar = "tt"
    Telugu = "te"
    Thai = "th"
    Tigrinya = "ti"
    Tonga = "to"
    Tshiluba = "lua"
    Tumbuka = "tum"
    Turkish = "tr"
    Turkmen = "tk"
    Twi = "tw"
    Uighur = "ug"
    Ukrainian = "uk"
    Urdu = "ur"
    Uzbek = "uz"
    Vietnamese = "vi"
    Welsh = "cy"
    Wolof = "wo"
    Xhosa = "xh"
    Yiddish = "yi"
    Yoruba = "yo"
    Zulu = "zu"
    Albanian = "sq"



#
#                py-google-search  Copyright (C) 2018  Javinator9889
#   This program comes with ABSOLUTELY NO WARRANTY; for details add the "-h" option.
#           This is free software, and you are welcome to redistribute it
#                 under certain conditions; type "-L" for details.
#

__version__ = "v1.0"
__version_code__ = 75


def print_ver_info():
    import sys

    print("pyGle {0}".format(__version__))
    print("Python {0}".format(sys.version.replace("\n", ' ')))



#
#                py-google-search  Copyright (C) 2018  Javinator9889                
#   This program comes with ABSOLUTELY NO WARRANTY; for details add the "-h" option.
#           This is free software, and you are welcome to redistribute it
#                 under certain conditions; type "-L" for details.
#

__google_base_url__ = "https://www.google.com/search?"
__google_url_modifiers__ = {
    "query": "q={0}",
    "in_order_displayed": "as_epq={0}",
    "or": "as_oq={0}",
    "exclude": "as_eq={0}",
    "number_of_results": "num={0}",
    "with_extension": "as_filetype={0}",
    "at_site": "as_sitesearch={0}",
    "with_time_limit": "as_qdr={0}",
    "with_time_limitation_for": {
        "day": 'd',
        "week": 'w',
        "month": 'm',
        "year": 'y'
    },
    "with_rights": "as_rights={0}",
    "with_rights_types": {
        "free_to_use": "(cc_publicdomain|cc_attribute|cc_sharealike|cc_noncommercial|cc_nonderived)",
        "free_to_use_com": "(cc_publicdomain|cc_attribute|cc_sharealike|cc_nonderived).-(cc_noncommercial)",
        "free_to_use_and_modify": "(cc_publicdomain|cc_attribute|cc_sharealike|cc_noncommercial).-(cc_nonderived)",
        "free_to_use_and_modify_com": "(cc_publicdomain|cc_attribute|cc_sharealike).-(cc_noncommercial|cc_nonderived)"
    },
    "with_licenses": ["cc_publicdomain", "cc_attribute", "cc_sharealike", "cc_noncommercial", "cc_nonderived"],
    "with_text_in_title": "allintitle%3A{0}",
    "with_text_in_body": "allintext%3A{0}",
    "with_text_in_url": "allinurl%3A{0}",
    "with_text_in_anchor": "allinanchor%3A{0}",
    "between_two_numbers": "{0}..{1}",
    "with_related_pages_to_a_doc": "as_rq={0}",
    "with_synonymous_searching": "~{0}",
    "with_defining": "define%3A{0}",
    "with_containing_two_terms": "{0}*{1}",
    "with_arithmetic_operation": {
        "sum": '+',
        "subtract": '-',
        "divide": '/',
        "multiply": '*',
        "power_of": '^',
        "percentage_of": '%'
    },
    "with_safe_active": "safe=active",
    "with_safe_deactivated": "safe=images",
    "with_linking_to_url": "as_lq={0}",
    "with_personalized_search": "pws=1",
    "with_no_personalized_search": "pws=0",
    "with_language_interface": "hl={0}",
    "with_results_language": "lr={0}",
    "with_languages": {
        "Arabic": "lang_ar",
        "Armenian": "lang_hy",
        "Belarusian": "lang_be",
        "Bulgarian": "lang_bg",
        "Catalan": "lang_ca",
        "Croatian": "lang_hr",
        "Czech": "lang_cs",
        "Danish": "lang_da",
        "Dutch": "lang_nl",
        "English": "lang_en",
        "Esperanto": "lang_eo",
        "Estonian": "lang_et",
        "Filipino": "lang_tl",
        "Finnish": "lang_fi",
        "French": "lang_fr",
        "German": "lang_de",
        "Greek": "lang_el",
        "Hebrew": "lang_iw",
        "Hungarian": "lang_hu",
        "Icelandic": "lang_is",
        "Indonesian": "lang_id",
        "Italian": "lang_it",
        "Japanese": "lang_ja",
        "Korean": "lang_ko",
        "Latvian": "lang_lv",
        "Lithuanian": "lang_lt",
        "Norwegian": "lang_no",
        "Persian": "lang_fa",
        "Polish": "lang_pl",
        "Portuguese": "lang_pt",
        "Romanian": "lang_ro",
        "Russian": "lang_ru",
        "Serbian": "lang_sr",
        "Slovak": "lang_sk",
        "Slovenian": "lang_sl",
        "Spanish": "lang_es",
        "Swedish": "lang_sv",
        "Thai": "lang_sv",
        "Turkish": "lang_tr",
        "Ukrainian": "lang_uk",
        "Vietnamese": "lang_vi",
        "Chinese Simplified": "lang_zh-CN",
        "Chinese Traditional": "lang_zh-TW"
    },
    "with_country_results": "cr=country{0}",
    "with_countries": {
        "Afghanistan": "AF",
        "Åland Islands": "AX",
        "Albania": "AL",
        "Algeria": "DZ",
        "American Samoa": "AS",
        "AndorrA": "AD",
        "Angola": "AO",
        "Anguilla": "AI",
        "Antarctica": "AQ",
        "Antigua and Barbuda": "AG",
        "Argentina": "AR",
        "Armenia": "AM",
        "Aruba": "AW",
        "Australia": "AU",
        "Austria": "AT",
        "Azerbaijan": "AZ",
        "Bahamas": "BS",
        "Bahrain": "BH",
        "Bangladesh": "BD",
        "Barbados": "BB",
        "Belarus": "BY",
        "Belgium": "BE",
        "Belize": "BZ",
        "Benin": "BJ",
        "Bermuda": "BM",
        "Bhutan": "BT",
        "Bolivia": "BO",
        "Bosnia and Herzegovina": "BA",
        "Botswana": "BW",
        "Bouvet Island": "BV",
        "Brazil": "BR",
        "British Indian Ocean Territory": "IO",
        "Brunei Darussalam": "BN",
        "Bulgaria": "BG",
        "Burkina Faso": "BF",
        "Burundi": "BI",
        "Cambodia": "KH",
        "Cameroon": "CM",
        "Canada": "CA",
        "Cape Verde": "CV",
        "Cayman Islands": "KY",
        "Central African Republic": "CF",
        "Chad": "TD",
        "Chile": "CL",
        "China": "CN",
        "Christmas Island": "CX",
        "Cocos (Keeling) Islands": "CC",
        "Colombia": "CO",
        "Comoros": "KM",
        "Congo": "CG",
        "Congo, Democratic Republic": "CD",
        "Cook Islands": "CK",
        "Costa Rica": "CR",
        "Cote D'Ivoire": "CI",
        "Croatia": "HR",
        "Cuba": "CU",
        "Cyprus": "CY",
        "Czech Republic": "CZ",
        "Denmark": "DK",
        "Djibouti": "DJ",
        "Dominica": "DM",
        "Dominican Republic": "DO",
        "Ecuador": "EC",
        "Egypt": "EG",
        "El Salvador": "SV",
        "Equatorial Guinea": "GQ",
        "Eritrea": "ER",
        "Estonia": "EE",
        "Ethiopia": "ET",
        "Falkland Islands (Malvinas)": "FK",
        "Faroe Islands": "FO",
        "Fiji": "FJ",
        "Finland": "FI",
        "France": "FR",
        "French Guiana": "GF",
        "French Polynesia": "PF",
        "French Southern Territories": "TF",
        "Gabon": "GA",
        "Gambia": "GM",
        "Georgia": "GE",
        "Germany": "DE",
        "Ghana": "GH",
        "Gibraltar": "GI",
        "Greece": "GR",
        "Greenland": "GL",
        "Grenada": "GD",
        "Guadeloupe": "GP",
        "Guam": "GU",
        "Guatemala": "GT",
        "Guernsey": "GG",
        "Guinea": "GN",
        "Guinea-Bissau": "GW",
        "Guyana": "GY",
        "Haiti": "HT",
        "Heard Island and Mcdonald Islands": "HM",
        "Holy See (Vatican City State)": "VA",
        "Honduras": "HN",
        "Hong Kong": "HK",
        "Hungary": "HU",
        "Iceland": "IS",
        "India": "IN",
        "Indonesia": "ID",
        "Iran": "IR",
        "Iraq": "IQ",
        "Ireland": "IE",
        "Isle of Man": "IM",
        "Israel": "IL",
        "Italy": "IT",
        "Jamaica": "JM",
        "Japan": "JP",
        "Jersey": "JE",
        "Jordan": "JO",
        "Kazakhstan": "KZ",
        "Kenya": "KE",
        "Kiribati": "KI",
        "Korea (North)": "KP",
        "Korea (South)": "KR",
        "Kosovo": "XK",
        "Kuwait": "KW",
        "Kyrgyzstan": "KG",
        "Laos": "LA",
        "Latvia": "LV",
        "Lebanon": "LB",
        "Lesotho": "LS",
        "Liberia": "LR",
        "Libyan Arab Jamahiriya": "LY",
        "Liechtenstein": "LI",
        "Lithuania": "LT",
        "Luxembourg": "LU",
        "Macao": "MO",
        "Macedonia": "MK",
        "Madagascar": "MG",
        "Malawi": "MW",
        "Malaysia": "MY",
        "Maldives": "MV",
        "Mali": "ML",
        "Malta": "MT",
        "Marshall Islands": "MH",
        "Martinique": "MQ",
        "Mauritania": "MR",
        "Mauritius": "MU",
        "Mayotte": "YT",
        "Mexico": "MX",
        "Micronesia": "FM",
        "Moldova": "MD",
        "Monaco": "MC",
        "Mongolia": "MN",
        "Montserrat": "MS",
        "Morocco": "MA",
        "Mozambique": "MZ",
        "Myanmar": "MM",
        "Namibia": "NA",
        "Nauru": "NR",
        "Nepal": "NP",
        "Netherlands": "NL",
        "Netherlands Antilles": "AN",
        "New Caledonia": "NC",
        "New Zealand": "NZ",
        "Nicaragua": "NI",
        "Niger": "NE",
        "Nigeria": "NG",
        "Niue": "NU",
        "Norfolk Island": "NF",
        "Northern Mariana Islands": "MP",
        "Norway": "NO",
        "Oman": "OM",
        "Pakistan": "PK",
        "Palau": "PW",
        "Palestinian Territory, Occupied": "PS",
        "Panama": "PA",
        "Papua New Guinea": "PG",
        "Paraguay": "PY",
        "Peru": "PE",
        "Philippines": "PH",
        "Pitcairn": "PN",
        "Poland": "PL",
        "Portugal": "PT",
        "Puerto Rico": "PR",
        "Qatar": "QA",
        "Reunion": "RE",
        "Romania": "RO",
        "Russian Federation": "RU",
        "Rwanda": "RW",
        "Saint Helena": "SH",
        "Saint Kitts and Nevis": "KN",
        "Saint Lucia": "LC",
        "Saint Pierre and Miquelon": "PM",
        "Saint Vincent and the Grenadines": "VC",
        "Samoa": "WS",
        "San Marino": "SM",
        "Sao Tome and Principe": "ST",
        "Saudi Arabia": "SA",
        "Senegal": "SN",
        "Serbia": "RS",
        "Montenegro": "ME",
        "Seychelles": "SC",
        "Sierra Leone": "SL",
        "Singapore": "SG",
        "Slovakia": "SK",
        "Slovenia": "SI",
        "Solomon Islands": "SB",
        "Somalia": "SO",
        "South Africa": "ZA",
        "South Georgia and the South Sandwich Islands": "GS",
        "Spain": "ES",
        "Sri Lanka": "LK",
        "Sudan": "SD",
        "Suriname": "SR",
        "Svalbard and Jan Mayen": "SJ",
        "Swaziland": "SZ",
        "Sweden": "SE",
        "Switzerland": "CH",
        "Syrian Arab Republic": "SY",
        "Taiwan, Province of China": "TW",
        "Tajikistan": "TJ",
        "Tanzania": "TZ",
        "Thailand": "TH",
        "Timor-Leste": "TL",
        "Togo": "TG",
        "Tokelau": "TK",
        "Tonga": "TO",
        "Trinidad and Tobago": "TT",
        "Tunisia": "TN",
        "Turkey": "TR",
        "Turkmenistan": "TM",
        "Turks and Caicos Islands": "TC",
        "Tuvalu": "TV",
        "Uganda": "UG",
        "Ukraine": "UA",
        "United Arab Emirates": "AE",
        "United Kingdom": "GB",
        "United States": "US",
        "United States Minor Outlying Islands": "UM",
        "Uruguay": "UY",
        "Uzbekistan": "UZ",
        "Vanuatu": "VU",
        "Venezuela": "VE",
        "Viet Nam": "VN",
        "Virgin Islands, British": "VG",
        "Virgin Islands, U.S.": "VI",
        "Wallis and Futuna": "WF",
        "Western Sahara": "EH",
        "Yemen": "YE",
        "Zambia": "ZM",
        "Zimbabwe": "ZW"
    },
    "with_document_county": "gl={0}",
    "with_extra_attributes": "tbs={0}",
    "with_search_between_two_dates": "cd_min:{0},cd_max:{1}",
    "with_search_between_two_dates_by_update_time": "cdr:1,sbd:1,cd_min:{0},cd_max:{1}",
    "with_search_by_update_time": "cdr:1,sbd:1",
    "with_searching_at_different_google_pages": "tbm={0}",
    "with_searching": {
        "books": "bks",
        "images": "isch",
        "news": "nws",
        "patents": "pts",
        "shops": "shop",
        "videos": "vid"
    },
    "with_starting_at_position": "start={0}",
    "with_image_params": {
        "color": {
            "red": "ic:specific,isc:red",
            "orange": "ic:specific,isc:orange",
            "yellow": "ic:specific,isc:yellow",
            "green": "ic:specific,isc:green",
            "teal": "ic:specific,isc:teel",
            "blue": "ic:specific,isc:blue",
            "purple": "ic:specific,isc:purple",
            "pink": "ic:specific,isc:pink",
            "white": "ic:specific,isc:white",
            "gray": "ic:specific,isc:gray",
            "black": "ic:specific,isc:black",
            "brown": "ic:specific,isc:brown"
        },
        "color_type": {
            "full-color": "ic:color",
            "black-and-white": "ic:gray",
            "transparent": "ic:trans"
        },
        "image_rights": {
            "labeled-for-reuse-with-modifications": "sur:fmc",
            "labeled-for-reuse": "sur:fc",
            "labeled-for-noncommercial-reuse-with-modification": "sur:fm",
            "labeled-for-noncommercial-reuse": "sur:f"
        },
        "image_size": {
            "large": "isz:l",
            "medium": "isz:m",
            "icon": "isz:i",
            ">400*300": "isz:lt,islt:qsvga",
            ">640*480": "isz:lt,islt:vga",
            ">800*600": "isz:lt,islt:svga",
            ">1024*768": "visz:lt,islt:xga",
            ">2MP": "isz:lt,islt:2mp",
            ">4MP": "isz:lt,islt:4mp",
            ">6MP": "isz:lt,islt:6mp",
            ">8MP": "isz:lt,islt:8mp",
            ">10MP": "isz:lt,islt:10mp",
            ">12MP": "isz:lt,islt:12mp",
            ">15MP": "isz:lt,islt:15mp",
            ">20MP": "isz:lt,islt:20mp",
            ">40MP": "isz:lt,islt:40mp",
            ">70MP": "isz:lt,islt:70mp"
        },
        "image_type": {
            "face": "itp:face",
            "photo": "itp:photo",
            "clip-art": "itp:clipart",
            "line-drawing": "itp:lineart",
            "animated": "itp:animated"
        },
        "aspect_ratio": {
            "tall": "iar:t",
            "square": "iar:s",
            "wide": "iar:w",
            "panoramic": "iar:xw"
        },
        "format": {
            "jpg": "ift:jpg",
            "gif": "ift:gif",
            "png": "ift:png",
            "bmp": "ift:bmp",
            "svg": "ift:svg",
            "webp": "webp",
            "ico": "ift:ico"
        }
    },
    "with_patents_params": {
        "with_search_at_patent_office": "ptso:{0}",
        "with_office_patents": {
            "EE.UU.": "us",
            "Europe": "ep",
            "International": "wo",
            "China": "cn",
            "Germany": "de",
            "Canada": "ca"
        },
        "with_patent_status": "ptss:{0}",
        "with_available_status": {
            "Applications": "a",
            "Issued patents": "g"
        },
        "with_patent_type": "ptst:{0}",
        "with_available_patent_types": {
            "Utility": "u",
            "Design": "d",
            "Plant": "pp",
            "Defensive Publication": "t",
            "Additional Improvement": "ai",
            "Statutory Invention Registration": "h"
        }
    },
    "with_shop_params": {
        "only_new_products": "new:1",
        "with_custom_price": "price:1",
        "with_max_price": "ppr_max:{0}",
        "with_min_price": "ppr_min:{0}",
        "sort_order_less_to_high": "p_ord:p",
        "sort_order_high_to_less": "p_ord:pd",
        "sort_order_reviews": "p_ord:rv"
    },
    "with_books_params": {
        "books_with_preview": "bkv:p",
        "google_ebooks": "bkv:e",
        "free_ebooks": "bkv:r",
        "with_searching_books": "bkt:b",
        "with_searching_news": "bkt:s",
        "with_searching_magazines": "bkt:m"
    },
    "with_video_params": {
        "duration": {
            "short": "dur:s",
            "medium": "dur:m",
            "long": "dur:l"
        },
        "high_quality": "hq:h",
        "with_subtitles": "cc:1"
    }
}

__user_agents__ = {
    "chrome": "Mozilla/5.0 (Windows NT 6.1) AppleWebKit/537.36 (KHTML, like Gecko) "
              "Chrome/41.0.2228.0 Safari/537.36",
    "edge": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/42.0.2311.135 Safari/537.36 Edge/12.246",
    "firefox": "Mozilla/5.0 (Windows NT 6.1; WOW64; rv:40.0) Gecko/20100101 Firefox/40.1",
    "safari": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_3) AppleWebKit/537.75.14 (KHTML, like Gecko) "
              "Version/7.0.3 Safari/7046A194A"
}



from setuptools import setup
from sys import version

from pyGle.version import __version__

if version < '3':
    raise RuntimeError("Python v3 at least needed")


def requirements() -> list:
    with open("pyGle/requirements.txt", 'r') as f:
        req = f.read().splitlines()
    return req


try:
    import codecs

    readme = codecs.open(filename="README.md", mode="r", encoding="utf-8")
    long_description = readme.read()
    readme.close()
except:
    long_description = ''


setup(
    name='g-pyGle',
    version=__version__,
    packages=['pyGle', 'pyGle.url', 'pyGle.values', 'pyGle.extractor'],
    url='https://github.com/Javinator9889/pyGle',
    license='GPL-3.0',
    author='Javinator9889',
    author_email='javialonso007@hotmail.es',
    description='A tool for searching the entire web with the Google technology',
    long_description=long_description,
    long_description_content_type="text/markdown",
    include_package_data=True,
    install_requires=requirements(),
    zip_safe=False,
    download_url="https://github.com/Javinator9889/pyGle/archive/master.zip",
    classifiers=[
            'Development Status :: 5 - Production/Stable',
            'Programming Language :: Python',
            'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
            'Natural Language :: English',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.1',
            'Programming Language :: Python :: 3.2',
            'Programming Language :: Python :: 3.3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
    ]
)



#
#                py-google-search  Copyright (C) 2018  Javinator9889                
#   This program comes with ABSOLUTELY NO WARRANTY; for details add the "-h" option.
#           This is free software, and you are welcome to redistribute it
#                 under certain conditions; type "-L" for details.
#
__all__ = ['PyGle']

from .url import GoogleSearch, URLBuilder
from .extractor import (BaseExtractor,
                        ImageExtractor,
                        SearchExtractor,
                        NewsExtractor,
                        VideoExtractor,
                        PatentExtractor,
                        ShopExtractor,
                        BookExtractor,
                        Future)
from .url.url_constants import __google_url_modifiers__
from .version import print_ver_info


class PyGle(GoogleSearch):
    def __init__(self, query: str = None, enable_history: bool = False, use_session_cookies: bool = False):
        super().__init__(query)
        self.__search_extractor = SearchExtractor(must_use_session=use_session_cookies,
                                                  with_history_enabled=enable_history)
        self.__books_extractor = BookExtractor(must_use_session=use_session_cookies,
                                               with_history_enabled=enable_history)
        self.__image_extractor = ImageExtractor(must_use_session=use_session_cookies,
                                                with_history_enabled=enable_history)
        self.__news_extractor = NewsExtractor(must_use_session=use_session_cookies,
                                              with_history_enabled=enable_history)
        self.__patents_extractor = PatentExtractor(must_use_session=use_session_cookies,
                                                   with_history_enabled=enable_history)
        self.__shops_extractor = ShopExtractor(must_use_session=use_session_cookies,
                                               with_history_enabled=enable_history)
        self.__videos_extractor = VideoExtractor(must_use_session=use_session_cookies,
                                                 with_history_enabled=enable_history)
        self.__session_cookies = use_session_cookies
        self.__history_enabled = enable_history

    @staticmethod
    def __torify():
        import socks
        import socket

        socks.set_default_proxy(proxy_type=socks.PROXY_TYPE_SOCKS5, addr="127.0.0.1", port=9050)
        socket.socket = socks.socksocket

    def doSearch(self, torify: bool = False) -> Future:
        url_builder_object = URLBuilder(self)
        ext = BaseExtractor(must_use_session=self.__session_cookies, with_history_enabled=self.__history_enabled)
        if self.search_at_different_pages:
            if self.search_at == __google_url_modifiers__["with_searching"]["books"]:
                ext = self.__books_extractor
            elif self.search_at == __google_url_modifiers__["with_searching"]["images"]:
                ext = self.__image_extractor
            elif self.search_at == __google_url_modifiers__["with_searching"]["news"]:
                ext = self.__news_extractor
            elif self.search_at == __google_url_modifiers__["with_searching"]["patents"]:
                ext = self.__patents_extractor
            elif self.search_at == __google_url_modifiers__["with_searching"]["shops"]:
                ext = self.__shops_extractor
            elif self.search_at == __google_url_modifiers__["with_searching"]["videos"]:
                ext = self.__videos_extractor
        else:
            ext = self.__search_extractor
        if torify:
            self.__torify()

        return ext.extract_url(url_builder_object)

    def pprintHistory(self):
        import pprint

        printer = pprint.PrettyPrinter(indent=4)
        if self.__history_enabled:
            history = self.getHistory()
            for key, value in history.items():
                if len(value) != 0:
                    print("Google {0} history".format(key))
                    printer.pprint(value)
        else:
            print("History is not enabled or no petition has been done yet")

    def getHistory(self) -> dict:
        return {"Books": self.__books_extractor.getHistory(),
                "Images": self.__image_extractor.getHistory(),
                "News": self.__news_extractor.getHistory(),
                "Patents": self.__patents_extractor.getHistory(),
                "Search": self.__search_extractor.getHistory(),
                "Shops": self.__shops_extractor.getHistory(),
                "Videos": self.__videos_extractor.getHistory()} if self.__history_enabled else {}

    def printOverallTime(self):
        if self.__history_enabled:
            history = self.getHistory()
            for key, value in history.items():
                if len(value) != 0:
                    print("Google {0} overall time".format(key))
                    if key == "Books":
                        self.__books_extractor.printOverallTime()
                    elif key == "Images":
                        self.__image_extractor.printOverallTime()
                    elif key == "News":
                        self.__news_extractor.printOverallTime()
                    elif key == "Patents":
                        self.__patents_extractor.printOverallTime()
                    elif key == "Search":
                        self.__search_extractor.printOverallTime()
                    elif key == "Shops":
                        self.__shops_extractor.printOverallTime()
                    elif key == "Videos":
                        self.__videos_extractor.printOverallTime()
        # self.__search_extractor.printOverallTime()

    @staticmethod
    def version():
        print_ver_info()


def main():
    import pprint

    printer = pprint.PrettyPrinter(indent=4)
    search = PyGle(enable_history=True)
    for i in range(5):
        search.withQuery("test")
        printer.pprint(search.doSearch().result())
        search.withQuery("test2")
        printer.pprint(search.doSearch().result())
    search.pprintHistory()
    search.printOverallTime()


if __name__ == '__main__':
    main()



#
#                py-google-search  Copyright (C) 2018  Javinator9889                
#   This program comes with ABSOLUTELY NO WARRANTY; for details add the "-h" option.
#           This is free software, and you are welcome to redistribute it
#                 under certain conditions; type "-L" for details.
#
from abc import ABCMeta


class AvailableCountries(metaclass=ABCMeta):
    Afghanistan = "AF"
    AlandIslands = "AX"
    Albania = "AL"
    Algeria = "DZ"
    AmericanSamoa = "AS"
    Andorra = "AD"
    Angola = "AO"
    Anguilla = "AI"
    Antarctica = "AQ"
    AntiguaAndBarbuda = "AG"
    Argentina = "AR"
    Armenia = "AM"
    Aruba = "AW"
    Australia = "AU"
    Austria = "AT"
    Azerbaijan = "AZ"
    Bahamas = "BS"
    Bahrain = "BH"
    Bangladesh = "BD"
    Barbados = "BB"
    Belarus = "BY"
    Belgium = "BE"
    Belize = "BZ"
    Benin = "BJ"
    Bermuda = "BM"
    Bhutan = "BT"
    Bolivia = "BO"
    BosniaAndHerzegovina = "BA"
    Botswana = "BW"
    BouvetIsland = "BV"
    Brazil = "BR"
    BritishIndianOceanTerritory = "IO"
    BruneiDarussalam = "BN"
    Bulgaria = "BG"
    BurkinaFaso = "BF"
    Burundi = "BI"
    Cambodia = "KH"
    Cameroon = "CM"
    Canada = "CA"
    CapeVerde = "CV"
    CaymanIslands = "KY"
    CentralAfricanRepublic = "CF"
    Chad = "TD"
    Chile = "CL"
    China = "CN"
    ChristmasIsland = "CX"
    CocosIslands = "CC"
    Colombia = "CO"
    Comoros = "KM"
    Congo = "CG"
    CongoDemocraticRepublic = "CD"
    CookIslands = "CK"
    CostaRica = "CR"
    CoteDIvoire = "CI"
    Croatia = "HR"
    Cuba = "CU"
    Cyprus = "CY"
    CzechRepublic = "CZ"
    Denmark = "DK"
    Djibouti = "DJ"
    Dominica = "DM"
    DominicanRepublic = "DO"
    Ecuador = "EC"
    Egypt = "EG"
    ElSalvador = "SV"
    EquatorialGuinea = "GQ"
    Eritrea = "ER"
    Estonia = "EE"
    Ethiopia = "ET"
    FalklandIslands = "FK"
    FaroeIslands = "FO"
    Fiji = "FJ"
    Finland = "FI"
    France = "FR"
    FrenchGuiana = "GF"
    FrenchPolynesia = "PF"
    FrenchSouthernTerritories = "TF"
    Gabon = "GA"
    Gambia = "GM"
    Georgia = "GE"
    Germany = "DE"
    Ghana = "GH"
    Gibraltar = "GI"
    Greece = "GR"
    Greenland = "GL"
    Grenada = "GD"
    Guadeloupe = "GP"
    Guam = "GU"
    Guatemala = "GT"
    Guernsey = "GG"
    Guinea = "GN"
    GuineaBissau = "GW"
    Guyana = "GY"
    Haiti = "HT"
    HeardIslandAndMcdonaldIslands = "HM"
    HolySeeVaticanCityState = "VA"
    Honduras = "HN"
    HongKong = "HK"
    Hungary = "HU"
    Iceland = "IS"
    India = "IN"
    Indonesia = "ID"
    Iran = "IR"
    Iraq = "IQ"
    Ireland = "IE"
    IsleOfMan = "IM"
    Israel = "IL"
    Italy = "IT"
    Jamaica = "JM"
    Japan = "JP"
    Jersey = "JE"
    Jordan = "JO"
    Kazakhstan = "KZ"
    Kenya = "KE"
    Kiribati = "KI"
    NorthKorea = "KP"
    SouthKorea = "KR"
    Kosovo = "XK"
    Kuwait = "KW"
    Kyrgyzstan = "KG"
    Laos = "LA"
    Latvia = "LV"
    Lebanon = "LB"
    Lesotho = "LS"
    Liberia = "LR"
    LibyanArabJamahiriya = "LY"
    Liechtenstein = "LI"
    Lithuania = "LT"
    Luxembourg = "LU"
    Macao = "MO"
    Macedonia = "MK"
    Madagascar = "MG"
    Malawi = "MW"
    Malaysia = "MY"
    Maldives = "MV"
    Mali = "ML"
    Malta = "MT"
    MarshallIslands = "MH"
    Martinique = "MQ"
    Mauritania = "MR"
    Mauritius = "MU"
    Mayotte = "YT"
    Mexico = "MX"
    Micronesia = "FM"
    Moldova = "MD"
    Monaco = "MC"
    Mongolia = "MN"
    Montserrat = "MS"
    Morocco = "MA"
    Mozambique = "MZ"
    Myanmar = "MM"
    Namibia = "NA"
    Nauru = "NR"
    Nepal = "NP"
    Netherlands = "NL"
    NetherlandsAntilles = "AN"
    NewCaledonia = "NC"
    NewZealand = "NZ"
    Nicaragua = "NI"
    Niger = "NE"
    Nigeria = "NG"
    Niue = "NU"
    NorfolkIsland = "NF"
    NorthernMarianaIslands = "MP"
    Norway = "NO"
    Oman = "OM"
    Pakistan = "PK"
    Palau = "PW"
    PalestinianTerritoryOccupied = "PS"
    Panama = "PA"
    PapuaNewGuinea = "PG"
    Paraguay = "PY"
    Peru = "PE"
    Philippines = "PH"
    Pitcairn = "PN"
    Poland = "PL"
    Portugal = "PT"
    PuertoRico = "PR"
    Qatar = "QA"
    Reunion = "RE"
    Romania = "RO"
    RussianFederation = "RU"
    Rwanda = "RW"
    SaintHelena = "SH"
    SaintKittsAndNevis = "KN"
    SaintLucia = "LC"
    SaintPierreAndMiquelon = "PM"
    SaintVincentAndTheGrenadines = "VC"
    Samoa = "WS"
    SanMarino = "SM"
    SaoTomeAndPrincipe = "ST"
    SaudiArabia = "SA"
    Senegal = "SN"
    Serbia = "RS"
    Montenegro = "ME"
    Seychelles = "SC"
    SierraLeone = "SL"
    Singapore = "SG"
    Slovakia = "SK"
    Slovenia = "SI"
    SolomonIslands = "SB"
    Somalia = "SO"
    SouthAfrica = "ZA"
    SouthGeorgiaAndTheSouthSandwichIslands = "GS"
    Spain = "ES"
    SriLanka = "LK"
    Sudan = "SD"
    Suriname = "SR"
    SvalbardAndJanMayen = "SJ"
    Swaziland = "SZ"
    Sweden = "SE"
    Switzerland = "CH"
    SyrianArabRepublic = "SY"
    TaiwanProvinceOfChina = "TW"
    Tajikistan = "TJ"
    Tanzania = "TZ"
    Thailand = "TH"
    TimorLeste = "TL"
    Togo = "TG"
    Tokelau = "TK"
    Tonga = "TO"
    TrinidadAndTobago = "TT"
    Tunisia = "TN"
    Turkey = "TR"
    Turkmenistan = "TM"
    TurksAndCaicosIslands = "TC"
    Tuvalu = "TV"
    Uganda = "UG"
    Ukraine = "UA"
    UnitedArabEmirates = "AE"
    UnitedKingdom = "GB"
    UnitedStates = "US"
    UnitedStatesMinorOutlyingIslands = "UM"
    Uruguay = "UY"
    Uzbekistan = "UZ"
    Vanuatu = "VU"
    Venezuela = "VE"
    Vietnam = "VN"
    VirginIslandsBritish = "VG"
    VirginIslandsUS = "VI"
    WallisAndFutuna = "WF"
    WesternSahara = "EH"
    Yemen = "YE"
    Zambia = "ZM"
    Zimbabwe = "ZW"
